@@ -41,15 +41,17 @@
 #include "qquickpopup_p_p.h"
 #include "qquickdeferredexecute_p_p.h"
 
-#include <QtCore/qregexp.h>
+#include <QtCore/qregularexpression.h>
 #include <QtCore/qabstractitemmodel.h>
+#include <QtCore/qglobal.h>
 #include <QtGui/qinputmethod.h>
 #include <QtGui/qguiapplication.h>
 #include <QtGui/qpa/qplatformtheme.h>
 #include <QtQml/qjsvalue.h>
 #include <QtQml/qqmlcontext.h>
 #include <QtQml/private/qlazilyallocated_p.h>
-#include <QtQml/private/qqmldelegatemodel_p.h>
+#include <private/qqmldelegatemodel_p.h>
+#include <QtQuick/private/qquickaccessibleattached_p.h>
 #include <QtQuick/private/qquickevents_p_p.h>
 #include <QtQuick/private/qquicktextinput_p.h>
 #include <QtQuick/private/qquickitemview_p.h>
@@ -59,7 +61,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \qmltype ComboBox
     \inherits Control
-    \instantiates QQuickComboBox
+//!     \instantiates QQuickComboBox
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols2-input
@@ -90,21 +92,7 @@ QT_BEGIN_NAMESPACE
     The following example demonstrates appending content to an editable
     combo box by reacting to the \l accepted signal.
 
-    \code
-    ComboBox {
-        editable: true
-        model: ListModel {
-            id: model
-            ListElement { text: "Banana" }
-            ListElement { text: "Apple" }
-            ListElement { text: "Coconut" }
-        }
-        onAccepted: {
-            if (find(editText) === -1)
-                model.append({text: editText})
-        }
-    }
-    \endcode
+    \snippet qtquickcontrols2-combobox-accepted.qml combobox
 
     \section1 ComboBox Model Roles
 
@@ -117,24 +105,20 @@ QT_BEGIN_NAMESPACE
 
     When using models that have multiple named roles, ComboBox must be configured
     to use a specific \l {textRole}{text role} for its \l {displayText}{display text}
-    and \l delegate instances.
+    and \l delegate instances. If you want to use a role of the model item
+    that corresponds to the text role, set \l valueRole. The \l currentValue
+    property and \l indexOfValue() method can then be used to get information
+    about those values.
 
-    \code
-    ComboBox {
-        textRole: "key"
-        model: ListModel {
-            ListElement { key: "First"; value: 123 }
-            ListElement { key: "Second"; value: 456 }
-            ListElement { key: "Third"; value: 789 }
-        }
-    }
-    \endcode
+    For example:
+
+    \snippet qtquickcontrols2-combobox-valuerole.qml file
 
     \note If ComboBox is assigned a data model that has multiple named roles, but
     \l textRole is not defined, ComboBox is unable to visualize it and throws a
     \c {ReferenceError: modelData is not defined}.
 
-    \sa {Customizing ComboBox}, {Input Controls}, {Focus Management in Qt Quick Controls 2}
+    \sa {Customizing ComboBox}, {Input Controls}, {Focus Management in Qt Quick Controls}
 */
 
 /*!
@@ -167,9 +151,22 @@ QT_BEGIN_NAMESPACE
     \qmlsignal void QtQuick.Controls::ComboBox::accepted()
 
     This signal is emitted when the \uicontrol Return or \uicontrol Enter key is pressed
-    on an \l editable combo box. If the confirmed string is not currently in the model,
-    the \l currentIndex will be set to \c -1 and the \c currentText will be updated
-    accordingly.
+    on an \l editable combo box.
+
+    You can handle this signal in order to add the newly entered
+    item to the model, for example:
+
+    \snippet qtquickcontrols2-combobox-accepted.qml combobox
+
+    Before the signal is emitted, a check is done to see if the string
+    exists in the model. If it does, \l currentIndex will be set to its index,
+    and \l currentText to the string itself.
+
+    After the signal has been emitted, and if the first check failed (that is,
+    the item did not exist), another check will be done to see if the item was
+    added by the signal handler. If it was, the \l currentIndex and
+    \l currentText are updated accordingly. Otherwise, they will be set to
+    \c -1 and \c "", respectively.
 
     \note If there is a \l validator set on the combo box, the signal will only be
           emitted if the input is in an acceptable state.
@@ -184,7 +181,7 @@ class QQuickComboBoxDelegateModel : public QQmlDelegateModel
 {
 public:
     explicit QQuickComboBoxDelegateModel(QQuickComboBox *combo);
-    QString stringValue(int index, const QString &role) override;
+    QVariant variantValue(int index, const QString &role) override;
 
 private:
     QQuickComboBox *combo = nullptr;
@@ -196,23 +193,23 @@ QQuickComboBoxDelegateModel::QQuickComboBoxDelegateModel(QQuickComboBox *combo)
 {
 }
 
-QString QQuickComboBoxDelegateModel::stringValue(int index, const QString &role)
+QVariant QQuickComboBoxDelegateModel::variantValue(int index, const QString &role)
 {
-    QVariant model = combo->model();
+    const QVariant model = combo->model();
     if (model.userType() == QMetaType::QVariantList) {
         QVariant object = model.toList().value(index);
         if (object.userType() == QMetaType::QVariantMap) {
             const QVariantMap data = object.toMap();
             if (data.count() == 1 && role == QLatin1String("modelData"))
-                return data.first().toString();
-            return data.value(role).toString();
+                return data.first();
+            return data.value(role);
         } else if (object.userType() == QMetaType::QObjectStar) {
             const QObject *data = object.value<QObject *>();
             if (data && role != QLatin1String("modelData"))
-                return data->property(role.toUtf8()).toString();
+                return data->property(role.toUtf8());
         }
     }
-    return QQmlDelegateModel::stringValue(index, role);
+    return QQmlDelegateModel::variantValue(index, role);
 }
 
 class QQuickComboBoxPrivate : public QQuickControlPrivate
@@ -235,6 +232,10 @@ public:
 
     void updateEditText();
     void updateCurrentText();
+    void updateCurrentValue();
+    void updateCurrentTextAndValue();
+
+    bool isValidIndex(int index) const;
 
     void acceptInput();
     QString tryComplete(const QString &inputText);
@@ -261,6 +262,11 @@ public:
     void cancelPopup();
     void executePopup(bool complete = false);
 
+    void itemImplicitWidthChanged(QQuickItem *item) override;
+    void itemImplicitHeightChanged(QQuickItem *item) override;
+
+    static void hideOldPopup(QQuickPopup *popup);
+
     bool flat = false;
     bool down = false;
     bool hasDown = false;
@@ -275,6 +281,8 @@ public:
     QString textRole;
     QString currentText;
     QString displayText;
+    QString valueRole;
+    QVariant currentValue;
     QQuickItem *pressedItem = nullptr;
     QQmlInstanceModel *delegateModel = nullptr;
     QQmlComponent *delegate = nullptr;
@@ -285,6 +293,7 @@ public:
         bool editable = false;
         bool accepting = false;
         bool allowComplete = false;
+        bool selectTextByMouse = false;
         Qt::InputMethodHints inputMethodHints = Qt::ImhNone;
         QString editText;
         QValidator *validator = nullptr;
@@ -395,13 +404,13 @@ void QQuickComboBoxPrivate::createdItem(int index, QObject *object)
     }
 
     if (index == currentIndex && !q->isEditable())
-        updateCurrentText();
+        updateCurrentTextAndValue();
 }
 
 void QQuickComboBoxPrivate::modelUpdated()
 {
     if (!extra.isAllocated() || !extra->accepting)
-        updateCurrentText();
+        updateCurrentTextAndValue();
 }
 
 void QQuickComboBoxPrivate::countChanged()
@@ -435,11 +444,11 @@ void QQuickComboBoxPrivate::updateEditText()
 void QQuickComboBoxPrivate::updateCurrentText()
 {
     Q_Q(QQuickComboBox);
-    QString text = q->textAt(currentIndex);
+    const QString text = q->textAt(currentIndex);
     if (currentText != text) {
         currentText = text;
         if (!hasDisplayText)
-           q->setAccessibleName(text);
+           q->maybeSetAccessibleName(text);
         emit q->currentTextChanged();
     }
     if (!hasDisplayText && displayText != text) {
@@ -450,6 +459,28 @@ void QQuickComboBoxPrivate::updateCurrentText()
         q->setEditText(currentText);
 }
 
+void QQuickComboBoxPrivate::updateCurrentValue()
+{
+    Q_Q(QQuickComboBox);
+    const QVariant value = q->valueAt(currentIndex);
+    if (currentValue == value)
+        return;
+
+    currentValue = value;
+    emit q->currentValueChanged();
+}
+
+void QQuickComboBoxPrivate::updateCurrentTextAndValue()
+{
+    updateCurrentText();
+    updateCurrentValue();
+}
+
+bool QQuickComboBoxPrivate::isValidIndex(int index) const
+{
+    return delegateModel && index >= 0 && index < delegateModel->count();
+}
+
 void QQuickComboBoxPrivate::acceptInput()
 {
     Q_Q(QQuickComboBox);
@@ -497,7 +528,7 @@ void QQuickComboBoxPrivate::setCurrentIndex(int index, Activation activate)
     emit q->currentIndexChanged();
 
     if (componentComplete)
-        updateCurrentText();
+        updateCurrentTextAndValue();
 
     if (activate)
         emit q->activated(index);
@@ -570,6 +601,8 @@ int QQuickComboBoxPrivate::match(int start, const QString &text, Qt::MatchFlags
     uint matchType = flags & 0x0F;
     bool wrap = flags & Qt::MatchWrap;
     Qt::CaseSensitivity cs = flags & Qt::MatchCaseSensitive ? Qt::CaseSensitive : Qt::CaseInsensitive;
+    QRegularExpression::PatternOptions options = flags & Qt::MatchCaseSensitive ? QRegularExpression::NoPatternOption
+                                                                                : QRegularExpression::CaseInsensitiveOption;
     int from = start;
     int to = q->count();
 
@@ -582,14 +615,19 @@ int QQuickComboBoxPrivate::match(int start, const QString &text, Qt::MatchFlags
                 if (t == text)
                     return idx;
                 break;
-            case Qt::MatchRegExp:
-                if (QRegExp(text, cs).exactMatch(t))
+            case Qt::MatchRegExp: {
+                QRegularExpression rx(QRegularExpression::anchoredPattern(text), options);
+                if (rx.match(t).hasMatch())
                     return idx;
                 break;
-            case Qt::MatchWildcard:
-                if (QRegExp(text, cs, QRegExp::Wildcard).exactMatch(t))
+            }
+            case Qt::MatchWildcard: {
+                QRegularExpression rx(QRegularExpression::wildcardToRegularExpression(text),
+                                      options);
+                if (rx.match(t).hasMatch())
                     return idx;
                 break;
+            }
             case Qt::MatchStartsWith:
                 if (t.startsWith(text, cs))
                     return idx;
@@ -724,6 +762,39 @@ void QQuickComboBoxPrivate::executePopup(bool complete)
         quickCompleteDeferred(q, popupName(), popup);
 }
 
+void QQuickComboBoxPrivate::itemImplicitWidthChanged(QQuickItem *item)
+{
+    Q_Q(QQuickComboBox);
+    QQuickControlPrivate::itemImplicitWidthChanged(item);
+    if (item == indicator)
+        emit q->implicitIndicatorWidthChanged();
+}
+
+void QQuickComboBoxPrivate::itemImplicitHeightChanged(QQuickItem *item)
+{
+    Q_Q(QQuickComboBox);
+    QQuickControlPrivate::itemImplicitHeightChanged(item);
+    if (item == indicator)
+        emit q->implicitIndicatorHeightChanged();
+}
+
+void QQuickComboBoxPrivate::hideOldPopup(QQuickPopup *popup)
+{
+    if (!popup)
+        return;
+
+    qCDebug(lcItemManagement) << "hiding old popup" << popup;
+
+    popup->setVisible(false);
+    popup->setParentItem(nullptr);
+#if QT_CONFIG(accessibility)
+    // Remove the item from the accessibility tree.
+    QQuickAccessibleAttached *accessible = accessibleAttached(popup);
+    if (accessible)
+        accessible->setIgnored(true);
+#endif
+}
+
 QQuickComboBox::QQuickComboBox(QQuickItem *parent)
     : QQuickControl(*(new QQuickComboBoxPrivate), parent)
 {
@@ -739,11 +810,12 @@ QQuickComboBox::QQuickComboBox(QQuickItem *parent)
 QQuickComboBox::~QQuickComboBox()
 {
     Q_D(QQuickComboBox);
+    d->removeImplicitSizeListener(d->indicator);
     if (d->popup) {
         // Disconnect visibleChanged() to avoid a spurious highlightedIndexChanged() signal
         // emission during the destruction of the (visible) popup. (QTBUG-57650)
         QObjectPrivate::disconnect(d->popup.data(), &QQuickPopup::visibleChanged, d, &QQuickComboBoxPrivate::popupVisibleChanged);
-        delete d->popup;
+        QQuickComboBoxPrivate::hideOldPopup(d->popup);
         d->popup = nullptr;
     }
 }
@@ -794,16 +866,21 @@ void QQuickComboBox::setModel(const QVariant& m)
     if (d->model == model)
         return;
 
-    if (QAbstractItemModel* aim = qvariant_cast<QAbstractItemModel *>(d->model))
-        QObjectPrivate::disconnect(aim, &QAbstractItemModel::dataChanged, d, &QQuickComboBoxPrivate::updateCurrentText);
-    if (QAbstractItemModel* aim = qvariant_cast<QAbstractItemModel *>(model))
-        QObjectPrivate::connect(aim, &QAbstractItemModel::dataChanged, d, &QQuickComboBoxPrivate::updateCurrentText);
+    if (QAbstractItemModel* aim = qvariant_cast<QAbstractItemModel *>(d->model)) {
+        QObjectPrivate::disconnect(aim, &QAbstractItemModel::dataChanged,
+            d, QOverload<>::of(&QQuickComboBoxPrivate::updateCurrentTextAndValue));
+    }
+    if (QAbstractItemModel* aim = qvariant_cast<QAbstractItemModel *>(model)) {
+        QObjectPrivate::connect(aim, &QAbstractItemModel::dataChanged,
+            d, QOverload<>::of(&QQuickComboBoxPrivate::updateCurrentTextAndValue));
+    }
 
     d->model = model;
     d->createDelegateModel();
+    emit countChanged();
     if (isComponentComplete()) {
         setCurrentIndex(count() > 0 ? 0 : -1);
-        d->updateCurrentText();
+        d->updateCurrentTextAndValue();
     }
     emit modelChanged();
 }
@@ -896,7 +973,7 @@ void QQuickComboBox::setCurrentIndex(int index)
 
     This property holds the text of the current item in the combo box.
 
-    \sa currentIndex, displayText, textRole
+    \sa currentIndex, displayText, textRole, editText
 */
 QString QQuickComboBox::currentText() const
 {
@@ -937,7 +1014,7 @@ void QQuickComboBox::setDisplayText(const QString &text)
         return;
 
     d->displayText = text;
-    setAccessibleName(text);
+    maybeSetAccessibleName(text);
     emit displayTextChanged();
 }
 
@@ -980,6 +1057,35 @@ void QQuickComboBox::setTextRole(const QString &role)
     emit textRoleChanged();
 }
 
+/*!
+    \since QtQuick.Controls 2.14 (Qt 5.14)
+    \qmlproperty string QtQuick.Controls::ComboBox::valueRole
+
+    This property holds the model role used for storing the value associated
+    with each item in the model.
+
+    For an example of how to use this property, see \l {ComboBox Model Roles}.
+
+    \sa model, currentValue
+*/
+QString QQuickComboBox::valueRole() const
+{
+    Q_D(const QQuickComboBox);
+    return d->valueRole;
+}
+
+void QQuickComboBox::setValueRole(const QString &role)
+{
+    Q_D(QQuickComboBox);
+    if (d->valueRole == role)
+        return;
+
+    d->valueRole = role;
+    if (isComponentComplete())
+        d->updateCurrentValue();
+    emit valueRoleChanged();
+}
+
 /*!
     \qmlproperty Component QtQuick.Controls::ComboBox::delegate
 
@@ -1047,12 +1153,22 @@ void QQuickComboBox::setIndicator(QQuickItem *indicator)
     if (!d->indicator.isExecuting())
         d->cancelIndicator();
 
-    delete d->indicator;
+    const qreal oldImplicitIndicatorWidth = implicitIndicatorWidth();
+    const qreal oldImplicitIndicatorHeight = implicitIndicatorHeight();
+
+    d->removeImplicitSizeListener(d->indicator);
+    QQuickControlPrivate::hideOldItem(d->indicator);
     d->indicator = indicator;
     if (indicator) {
         if (!indicator->parentItem())
             indicator->setParentItem(this);
+        d->addImplicitSizeListener(indicator);
     }
+
+    if (!qFuzzyCompare(oldImplicitIndicatorWidth, implicitIndicatorWidth()))
+        emit implicitIndicatorWidthChanged();
+    if (!qFuzzyCompare(oldImplicitIndicatorHeight, implicitIndicatorHeight()))
+        emit implicitIndicatorHeightChanged();
     if (!d->indicator.isExecuting())
         emit indicatorChanged();
 }
@@ -1089,7 +1205,7 @@ void QQuickComboBox::setPopup(QQuickPopup *popup)
 
     if (d->popup) {
         QObjectPrivate::disconnect(d->popup.data(), &QQuickPopup::visibleChanged, d, &QQuickComboBoxPrivate::popupVisibleChanged);
-        delete d->popup;
+        QQuickComboBoxPrivate::hideOldPopup(d->popup);
     }
     if (popup) {
         QQuickPopupPrivate::get(popup)->allowVerticalFlip = true;
@@ -1229,7 +1345,7 @@ void QQuickComboBox::setEditable(bool editable)
 
     This property holds the text in the text field of an editable combo box.
 
-    \sa editable
+    \sa editable, currentText, displayText
 */
 QString QQuickComboBox::editText() const
 {
@@ -1363,6 +1479,121 @@ bool QQuickComboBox::hasAcceptableInput() const
     return d->contentItem && d->contentItem->property("acceptableInput").toBool();
 }
 
+/*!
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+    \qmlproperty real QtQuick.Controls::ComboBox::implicitIndicatorWidth
+    \readonly
+
+    This property holds the implicit indicator width.
+
+    The value is equal to \c {indicator ? indicator.implicitWidth : 0}.
+
+    This is typically used, together with \l {Control::}{implicitContentWidth} and
+    \l {Control::}{implicitBackgroundWidth}, to calculate the \l {Item::}{implicitWidth}.
+
+    \sa implicitIndicatorHeight
+*/
+qreal QQuickComboBox::implicitIndicatorWidth() const
+{
+    Q_D(const QQuickComboBox);
+    if (!d->indicator)
+        return 0;
+    return d->indicator->implicitWidth();
+}
+
+/*!
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+    \qmlproperty real QtQuick.Controls::ComboBox::implicitIndicatorHeight
+    \readonly
+
+    This property holds the implicit indicator height.
+
+    The value is equal to \c {indicator ? indicator.implicitHeight : 0}.
+
+    This is typically used, together with \l {Control::}{implicitContentHeight} and
+    \l {Control::}{implicitBackgroundHeight}, to calculate the \l {Item::}{implicitHeight}.
+
+    \sa implicitIndicatorWidth
+*/
+qreal QQuickComboBox::implicitIndicatorHeight() const
+{
+    Q_D(const QQuickComboBox);
+    if (!d->indicator)
+        return 0;
+    return d->indicator->implicitHeight();
+}
+
+/*!
+    \readonly
+    \since QtQuick.Controls 2.14 (Qt 5.14)
+    \qmlproperty string QtQuick.Controls::ComboBox::currentValue
+
+    This property holds the value of the current item in the combo box.
+
+    For an example of how to use this property, see \l {ComboBox Model Roles}.
+
+    \sa currentIndex, currentText, valueRole
+*/
+QVariant QQuickComboBox::currentValue() const
+{
+    Q_D(const QQuickComboBox);
+    return d->currentValue;
+}
+
+QVariant QQuickComboBox::valueAt(int index) const
+{
+    Q_D(const QQuickComboBox);
+    if (!d->isValidIndex(index))
+        return QVariant();
+
+    const QString effectiveValueRole = d->valueRole.isEmpty() ? QStringLiteral("modelData") : d->valueRole;
+    return d->delegateModel->variantValue(index, effectiveValueRole);
+}
+
+/*!
+    \since QtQuick.Controls 2.14 (Qt 5.14)
+    \qmlmethod int QtQuick.Controls::ComboBox::indexOfValue(object value)
+
+    Returns the index of the specified \a value, or \c -1 if no match is found.
+
+    For an example of how to use this method, see \l {ComboBox Model Roles}.
+
+    \sa find(), currentValue, currentIndex, valueRole
+*/
+int QQuickComboBox::indexOfValue(const QVariant &value) const
+{
+    for (int i = 0; i < count(); ++i) {
+        const QVariant ourValue = valueAt(i);
+        if (value == ourValue)
+            return i;
+    }
+    return -1;
+}
+
+/*!
+    \since QtQuick.Controls 2.15 (Qt 5.15)
+    \qmlproperty bool QtQuick.Controls::ComboBox::selectTextByMouse
+
+    This property holds whether the text field for an editable ComboBox
+    can be selected with the mouse.
+
+    The default value is \c false.
+*/
+bool QQuickComboBox::selectTextByMouse() const
+{
+    Q_D(const QQuickComboBox);
+    return d->extra.isAllocated() ? d->extra->selectTextByMouse : false;
+}
+
+void QQuickComboBox::setSelectTextByMouse(bool canSelect)
+{
+    Q_D(QQuickComboBox);
+    if (canSelect == selectTextByMouse())
+        return;
+
+    d->extra.value().selectTextByMouse = canSelect;
+    emit selectTextByMouseChanged();
+}
 /*!
     \qmlmethod string QtQuick.Controls::ComboBox::textAt(int index)
 
@@ -1374,20 +1605,15 @@ bool QQuickComboBox::hasAcceptableInput() const
 QString QQuickComboBox::textAt(int index) const
 {
     Q_D(const QQuickComboBox);
-    if (!d->delegateModel || index < 0 || index >= d->delegateModel->count())
+    if (!d->isValidIndex(index))
         return QString();
 
-    QString text;
-    QObject *object = d->delegateModel->object(index);
-    if (object) {
-        text = d->delegateModel->stringValue(index, d->textRole.isEmpty() ? QStringLiteral("modelData") : d->textRole);
-        d->delegateModel->release(object);
-    }
-    return text;
+    const QString effectiveTextRole = d->textRole.isEmpty() ? QStringLiteral("modelData") : d->textRole;
+    return d->delegateModel->stringValue(index, effectiveTextRole);
 }
 
 /*!
-    \qmlmethod int QtQuick.Controls::ComboBox::find(string text, flags = Qt.MatchExactly)
+    \qmlmethod int QtQuick.Controls::ComboBox::find(string text, enumeration flags)
 
     Returns the index of the specified \a text, or \c -1 if no match is found.
 
@@ -1475,8 +1701,19 @@ bool QQuickComboBox::eventFilter(QObject *object, QEvent *event)
         break;
     }
     case QEvent::FocusOut:
-        d->hidePopup(false);
-        setPressed(false);
+        if (qGuiApp->focusObject() != this && (!d->popup || !d->popup->hasActiveFocus())) {
+            // Only close the popup if focus was transferred somewhere else
+            // than to the popup or the popup button (which normally means that
+            // the user clicked on the popup button to open it, not close it).
+            d->hidePopup(false);
+            setPressed(false);
+
+            // The focus left the text field, so if the edit text matches an item in the model,
+            // change our currentIndex to that. This matches widgets' behavior.
+            const int indexForEditText = find(d->extra.value().editText, Qt::MatchFixedString);
+            if (indexForEditText > -1)
+                setCurrentIndex(indexForEditText);
+        }
         break;
 #if QT_CONFIG(im)
     case QEvent::InputMethod:
@@ -1502,8 +1739,14 @@ void QQuickComboBox::focusOutEvent(QFocusEvent *event)
 {
     Q_D(QQuickComboBox);
     QQuickControl::focusOutEvent(event);
-    d->hidePopup(false);
-    setPressed(false);
+
+    if (qGuiApp->focusObject() != d->contentItem && (!d->popup || !d->popup->hasActiveFocus())) {
+        // Only close the popup if focus was transferred
+        // somewhere else than to the popup or the inner line edit (which is
+        // normally done from QQuickComboBox::focusInEvent).
+        d->hidePopup(false);
+        setPressed(false);
+    }
 }
 
 #if QT_CONFIG(im)
@@ -1624,6 +1867,14 @@ void QQuickComboBox::wheelEvent(QWheelEvent *event)
 }
 #endif
 
+bool QQuickComboBox::event(QEvent *e)
+{
+    Q_D(QQuickComboBox);
+    if (e->type() == QEvent::LanguageChange)
+        d->updateCurrentTextAndValue();
+    return QQuickControl::event(e);
+}
+
 void QQuickComboBox::componentComplete()
 {
     Q_D(QQuickComboBox);
@@ -1639,7 +1890,7 @@ void QQuickComboBox::componentComplete()
         if (!d->hasCurrentIndex && d->currentIndex == -1)
             setCurrentIndex(0);
         else
-            d->updateCurrentText();
+            d->updateCurrentTextAndValue();
     }
 }
 
@@ -1690,12 +1941,12 @@ void QQuickComboBox::localeChange(const QLocale &newLocale, const QLocale &oldLo
 
 QFont QQuickComboBox::defaultFont() const
 {
-    return QQuickControlPrivate::themeFont(QPlatformTheme::ComboMenuItemFont);
+    return QQuickTheme::font(QQuickTheme::ComboBox);
 }
 
 QPalette QQuickComboBox::defaultPalette() const
 {
-    return QQuickControlPrivate::themePalette(QPlatformTheme::ComboBoxPalette);
+    return QQuickTheme::palette(QQuickTheme::ComboBox);
 }
 
 #if QT_CONFIG(accessibility)
@@ -1710,7 +1961,7 @@ void QQuickComboBox::accessibilityActiveChanged(bool active)
     QQuickControl::accessibilityActiveChanged(active);
 
     if (active) {
-        setAccessibleName(d->hasDisplayText ? d->displayText : d->currentText);
+        maybeSetAccessibleName(d->hasDisplayText ? d->displayText : d->currentText);
         setAccessibleProperty("editable", isEditable());
     }
 }
