@@ -37,6 +37,7 @@
 #include "qquickdrawer_p.h"
 #include "qquickdrawer_p_p.h"
 #include "qquickpopupitem_p_p.h"
+#include "qquickpopuppositioner_p_p.h"
 
 #include <QtGui/qstylehints.h>
 #include <QtGui/private/qguiapplication_p.h>
@@ -50,7 +51,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \qmltype Drawer
     \inherits Popup
-    \instantiates QQuickDrawer
+//!     \instantiates QQuickDrawer
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols2-navigation
@@ -67,9 +68,9 @@ QT_BEGIN_NAMESPACE
     drawer is then opened by \e "dragging" it out from the left edge of the
     window.
 
-    \code
-    import QtQuick 2.7
-    import QtQuick.Controls 2.0
+    \code \QtMinorVersion
+    import QtQuick 2.\1
+    import QtQuick.Controls 2.\1
 
     ApplicationWindow {
         id: window
@@ -97,9 +98,9 @@ QT_BEGIN_NAMESPACE
     Drawer can be configured to cover only part of its window edge. The following example
     illustrates how Drawer can be positioned to appear below a window header:
 
-    \code
-    import QtQuick 2.7
-    import QtQuick.Controls 2.0
+    \code \QtMinorVersion
+    import QtQuick 2.\1
+    import QtQuick.Controls 2.\1
 
     ApplicationWindow {
         id: window
@@ -123,9 +124,9 @@ QT_BEGIN_NAMESPACE
     In the image above, the application's contents are \e "pushed" across the
     screen. This is achieved by applying a translation to the contents:
 
-    \code
-    import QtQuick 2.7
-    import QtQuick.Controls 2.1
+    \code \QtMinorVersion
+    import QtQuick 2.\1
+    import QtQuick.Controls 2.\1
 
     ApplicationWindow {
         id: window
@@ -171,6 +172,14 @@ QT_BEGIN_NAMESPACE
     \sa SwipeView, {Customizing Drawer}, {Navigation Controls}, {Popup Controls}
 */
 
+class QQuickDrawerPositioner : public QQuickPopupPositioner
+{
+public:
+    QQuickDrawerPositioner(QQuickDrawer *drawer) : QQuickPopupPositioner(drawer) { }
+
+    void reposition() override;
+};
+
 qreal QQuickDrawerPrivate::offsetAt(const QPointF &point) const
 {
     qreal offset = positionAt(point) - position;
@@ -203,14 +212,27 @@ qreal QQuickDrawerPrivate::positionAt(const QPointF &point) const
     }
 }
 
-void QQuickDrawerPrivate::reposition()
+QQuickPopupPositioner *QQuickDrawerPrivate::getPositioner()
 {
     Q_Q(QQuickDrawer);
-    QQuickWindow *window = q->window();
+    if (!positioner)
+        positioner = new QQuickDrawerPositioner(q);
+    return positioner;
+}
+
+void QQuickDrawerPositioner::reposition()
+{
+    if (m_positioning)
+        return;
+
+    QQuickDrawer *drawer = static_cast<QQuickDrawer*>(popup());
+    QQuickWindow *window = drawer->window();
     if (!window)
         return;
 
-    switch (edge) {
+    const qreal position = drawer->position();
+    QQuickItem *popupItem = drawer->popupItem();
+    switch (drawer->edge()) {
     case Qt::LeftEdge:
         popupItem->setX((position - 1.0) * popupItem->width());
         break;
@@ -225,7 +247,7 @@ void QQuickDrawerPrivate::reposition()
         break;
     }
 
-    QQuickPopupPrivate::reposition();
+    QQuickPopupPositioner::reposition();
 }
 
 void QQuickDrawerPrivate::showOverlay()
@@ -460,6 +482,8 @@ bool QQuickDrawerPrivate::handleMove(QQuickItem *item, const QPointF &point, ulo
 
 bool QQuickDrawerPrivate::handleRelease(QQuickItem *item, const QPointF &point, ulong timestamp)
 {
+    if (pressPoint.isNull())
+        return false;
     if (!popupItem->keepMouseGrab() && !popupItem->keepTouchGrab()) {
         velocityCalculator.reset();
         return QQuickPopupPrivate::handleRelease(item, point, timestamp);
